! test_graph_csr.f90 --- black box test for csr data structure
!
! Copyright  (C)  2011  Thomas Jahns <Thomas.Jahns@gmx.net>
!
! Version: 1.0
! Keywords: CSR compressed sparse row
! Author: Thomas Jahns <Thomas.Jahns@gmx.net>
! Maintainer: Thomas Jahns <Thomas.Jahns@gmx.net>
! URL: https://www.dkrz.de/redmine/projects/show/scales-ppm
!
! Redistribution and use in source and binary forms, with or without
! modification, are  permitted provided that the following conditions are
! met:
!
! Redistributions of source code must retain the above copyright notice,
! this list of conditions and the following disclaimer.
!
! Redistributions in binary form must reproduce the above copyright
! notice, this list of conditions and the following disclaimer in the
! documentation and/or other materials provided with the distribution.
!
! Neither the name of the DKRZ GmbH nor the names of its contributors
! may be used to endorse or promote products derived from this software
! without specific prior written permission.
!
! THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
! IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
! TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
! PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
! OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
! EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
! PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
! PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
! LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
! NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
! SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
!
PROGRAM test_graph_csr
  USE ppm_base, ONLY: ppm_default_comm, abort_ppm
  USE ppm_extents, ONLY: extent
  USE ppm_graph_csr, ONLY: graph_csr, build_graph, build_graph_mt
  USE ppm_random, ONLY: initialize_irand, irand
#ifdef _OPENMP
  USE omp_lib, ONLY: omp_get_thread_num
#endif
  IMPLICIT NONE

  INTEGER, PARAMETER :: max_nodes = 10000
  TYPE(graph_csr) :: graph
  LOGICAL, ALLOCATABLE :: adj(:,:)
  INTEGER :: e, i, j, m, n, ofs, rand_seed
  INTEGER :: tid
  LOGICAL :: p
  ! CHARACTER(len=10) :: adj_fm

!$omp parallel private(tid, i, rand_seed)
#ifdef _OPENMP
  tid = omp_get_thread_num()
#else
  tid = 0
#endif
  CALL initialize_irand(ppm_default_comm, 0, rand_seed)
!$omp critical
  PRINT '(2(a,i0))', 'thread id=', tid, ', random seed=', rand_seed
!$omp end critical

!$omp master
  ofs = HUGE(ofs)
  DO WHILE (ofs > HUGE(ofs) - max_nodes)
    ofs = irand()
  END DO
  n = MODULO(irand(), max_nodes) + ofs
  ALLOCATE(adj(ofs:n, ofs:n))
!$omp end master
!$omp barrier
  ! first test undirected graph
!$omp do firstprivate(n) private(p)
  DO j = ofs, n
    adj(j, j) = MODULO(irand(), 2) == 1
    DO i = j + 1, n
      p = MODULO(irand(), 2) == 1
      adj(i, j) = p
      adj(j, i) = p
    END DO
  END DO
!$omp end do nowait
  CALL build_graph(graph, adj, assert_undirected = .TRUE., &
       node_offset=ofs)

  ! WRITE (adj_fmt, '("(",i0,"l)")') n
  ! PRINT adj_fmt, adj
  ! PRINT *, graph%edges_of_vtx
  ! PRINT *, graph%edges
!$omp do private(m, e)
  DO j = ofs, n
    m = graph%edges_of_vtx(j + 1) - 1
    DO e = graph%edges_of_vtx(j), m
      IF (.NOT. adj(graph%edges(e), j)) THEN
        CALL abort_ppm('graph does not contain requested edge', &
             __FILE__, &
             __LINE__)
      END IF
    END DO
  END DO
!$omp end do

  CALL build_graph_mt(graph, (/ extent(1,5), extent(1,5) /))
!$omp end parallel

END PROGRAM test_graph_csr
!
! Local Variables:
! license-project-url: "https://www.dkrz.de/redmine/projects/show/scales-ppm"
! license-default: "bsd"
! End:
